#ifndef _Externals_h
#define _Externals_h 1
#include "Constants.h"

//#define STOP_CFG  -1  // used to mark end of table Apr08 999999 to -1
// STOP_CFG not used anymore because of unsignedTermCompression + STOP_TERM
#define STOP_TERM  0  // used to mark end of table (valid=1..81)

// projection/mapping of 62 terms to 13 input terms
// map13: /* 1 */ zz z+ z- +z -z +- -+ ++ -- /* 10 */ z - z^2 + 
extern const int map62[62+1];

// mapping of 81 excitation terms to 13 output terms
extern const int map81[81+1];
      
struct prm {
  /* This structure contains all of the globally visible information
     concerning runtime CCM Parameters set by the users either in the
     code itself, as command line instructions, using the GUI, or in
     some input file. 
     
     This is distinct from Control Parameters which simply tell the 
     code how much output to make, what filenames to use. 
  */
  int n;                // SUBn-m (n) 
  int m;                // SUBn-m (m) 
  int btLength;         // Joerg Oct05: length of bounding triangle? == m
  int dim;              // Dimension Of Lattice
  int nExpect;          // number of Expectation values
  // float s;           // Global Quantum Spin Number (removed Joerg Sep05)
  int s2;               // twofold of Global Quantum Spin Number
  int bonds;            // number of `bonds'
  
};
extern struct prm Params;

struct ctrl {
  /* 
     Control Parameters which simply tell the 
     code how much output to make, what filenames to use. 
  */
  int output;                           // Level Of Output
  int greatExpectations;                // find generalised expectation values?
  int excitations;                      // find excitations (==1) or not (==0)?
  int file;                             // save CCM equations to file
  int startx;				// start vector JS1004
  char name[200];                       // Root File Name                
};
extern struct ctrl Control;


// ToDo: replace by [].{x,y,z,s2} to get rid of MAX_UNIT_CELL
struct UC {
  /* this structure completely defines the unit cell */

  int number;             // number of sites in unit cell (UC)
  double x[1+MAX_UNIT_CELL];   // number of sites in UC
  double y[1+MAX_UNIT_CELL];   // number of sites in 
  double z[1+MAX_UNIT_CELL];   // number of sites in 
  // float s[1+MAX_UNIT_CELL]; // `local' quantum spin number for each site in UC
  int  s2[1+MAX_UNIT_CELL]; // 2x `local' quantum spin number for each site in UC
  // [0] is not used, using [1..MAX_UNIT_CELL]
};

extern struct UC UnitCell;

#if 0
struct sub {
  /* which terms from the unit cell to determine the
     sublattice for */
  int number;
  int cell[1+MAX_UNIT_CELL];
};
extern struct sub sublatticeMagnetisation;
#endif

struct neighbours {

  /* this is an interesting structure because it defines the 
     nearest neighbour structure for each of the elements of 
     the unit cell.

     Essentially each unit cell has however many nearest-neighbours,
     some of which in within the unit cell itself, and some which
     are in neigbouring cells.

     This information is most easily coded in by the user, or
     by writing a simple bit of code to determine the neighbours
     matrix of the lattice or Hamiltonian.

     Note that this also offers the possiblity of defining
     nearest-neighbours, next-nearest-neighbours, next-next-...
     and so on.

  */

  int number[1+MAX_UNIT_CELL]; /* number of nearest-neighbours for
				each site in the UC */
  int    x[1+MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS]; // TRANSLATION x bravais
  int    y[1+MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS]; // TRANSLATION y bravais
  int    z[1+MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS]; // TRANSLATION z bravais
  int cell[1+MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS]; // which site in UC
  
};
extern struct neighbours LatticeNeighbours;

//struct bravais {
//  /* structure defines the bavais lattice */
//  double a[4];
//  double b[4];
//  double c[4];
//};  // replaced by array[3][3]

extern double BravaisLattice[3][3];
extern double inversBravaisLattice[3][3];

struct symmetries {

  /* this structure contains all of the information concerning
     the symmetries of the lattice, Hamiltonian, unit cell etc. */

  int number;     // number of symmetries in lattice, Hamiltonian etc

  // JS0903: what was it good for?
  //int switchFunction(int switchVariable, 
  //		     double x1,  double y1,  double z1,
  //		     double &x2, double &y2, double &z2);

  // JS1003: new index 0..2 (old: 0=unused, 1..3)
  // double R[MAX_NUM_SYM][4][5]; // ROTATION MATRIX[4][4] + TRANSLATION[1][4]
  double R[MAX_NUM_SYM][3][4]; // ROTATION MATRIX[3][3] + TRANSLATION[0][3]

  // JS: translation, used in dimer_bonds_square_lattice.ccm,
  //     lc_mf.ccm, srcubo_spiral.ccm, ...
  //     reflection arround bond center??
  // double x[MAX_NUM_SYM];       // NON-BRAVAIS LATTICE VECTOR.
  // double y[MAX_NUM_SYM];
  // double z[MAX_NUM_SYM];
  // Jun08: moved to R[1..MAX_NUM_SYM][1..3][4] 

};

extern struct symmetries LatticeSymmetries;

/* 
   This structure describes a particular spin configuration by storing 
   the information in an array. The size of the (integer) numbers
   becomes too long for saving as a standard `int' -- or even `long int.'
   
   Each integer element gives the information on 20 lattices sites
   each of which has a spin of quantum spin number Params.s or
   UnitCell.s[]
 */
struct spinConfigs { int descriptor[DESCR_SIZE/* +3 */]; };

extern struct spinConfigs *spinConfigurations;
extern struct spinConfigs *excitedSpinConfigurations;
extern int         NumberConfigurations;
extern int         NumberExcitedConfigurations;

// #define MAX_TERMS62  62   // was 200 (Sep05 Joerg)
// #define MAX_TERMS13  13   // was 17, partly 200
// #define MAX_BONDS    10   // 2to10 Sep05, obsolete JS Feb10

//     terms used range=typeArray[1..13][l]=1..62 (see input.C)
// also used in finding generalised expectation values (May08)
struct hamiltonian_defn {

  int evaluate;
  double coefficient[1+MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS+1]; // JS1002 [+1]

  int k_to_m_number[1+MAX_UNIT_CELL];
  int k_to_m[1+MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS+1][4];  // [-,x,y,z,cell] 

  int m_to_k_number[1+MAX_UNIT_CELL];                
  int m_to_k[1+MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS+1][4];  // [-,x,y,z,cell]
};

//  array index term moved to simplify code (Joerg Sep05)
// read in from data-file
// Jun08 Joerg: simplified to one reference
// Feb10 Joerg: static array replaced by pointer + dynamic memory allocation
extern struct hamiltonian_defn (*Hamiltonian)[13+1];
extern struct hamiltonian_defn (*expectation)[13+1];

// other global variables

extern int sizeLUT;          // size of lookUpTable
extern int (*lookUpTable)[3];  // idx to {x,y,z}
extern int *BTarray;           // {x,y,z} to idx (bounding triangle)

/* obtain */

extern int binary_max[31];
// extern int trimary_max[MAX_SUBN+1];

// files

// extern char terms_fname[80]; /* term file fileName */
// extern int  terms_shm_fd;    /* term file descriptor, if SHM used */
// extern FILE *terms_fp;       /* term file */
// extern FILE *fp_gs;   /* gs-terms */
extern FILE *fp1;     /* input.ccm */
extern FILE *fp2;     /* output.results */
// extern FILE *fp_exp;     /* expectation.results */  obsolete! Jun08 id=3
extern FILE *fp_exc;     /* CCM excitation equations, ToDo: replace by id=4 */
/*
 * - linux has no problems by using all free memory as IO cache
 *   and give it back on demand, so we dont need lot of extra buffer
 * - solaris has a buffer cache of fixed size, this is bad and
 *   can cause excessiv disk access where there is enough free memory 
 *   or dont give back memory when other jobs need it
 * - so the user can choose full buffering to reduce disk access to 0
 *   
 */
// extern char *fp_buf; /* read/write buffer may be bigger than the file */
        
// solve variables.

extern double maximum;
extern double minimum;
extern double increment;

// restriction on choice of clusters.

extern int restrictionClusters;

// direction bonds or not?

extern int directional;

// solution strategy

extern int solutionStrategy;

// parallel variables

extern int myid;
extern int npe;
#endif
